''' Vogliamo disegnare i dati presenti in un file di testo su un grafico.
    Il file contiene su ciascuna riga, seguiti da 1 virgola e/o 0 o più spazi o accapi, 
    gruppi di 5 valori interi che rappresentano per ciascun dato:
        - larghezza
        - alezza
        - canale R del colore
        - canale G del colore
        - canale B del colore
    Ciascuna riga contiene almeno un gruppo di 5 interi positivi relativi ad un dato da plottare.
    Per ciascun dato dobbiamo disegnare un rettangolo del colore indicato e di dimensioni indicate

    Realizzate la funzione ex2(file_dati, file_png, spaziatura) che:
    - legge i dati dal file file_dati
    - costruisce una immagine in formato PNG e la salva nel file file_png
    - ritorna le dimensioni larghezza,altezza dell'immagine

    L'immagine deve avere sfondo nero e visualizzare tutti i dati come segue:
    - l'argomento spaziatura indica il numero di pixel da usare per il bordo esterno
      e la spaziatura minima in orizzontale tra i rettangoli ed in verticale tra le righe
    - ciascun dato è rappresentato da un rettangolo descritto da una quintupla
    - i dati contenuti su ciascuna riga del file devono essere disegnati, centrati verticalmente, su una riga della immagine
    - i rettangoli devono essere equidistanti orizzontalmente l'uno dall'altro 
        con una distanza minima di 'spaziatura' pixel tra un rettangolo ed il seguente
        in modo che tutti i primi rettangoli si trovino a 'spaziatura' pixel dal bordo sinistro
        e tutti gli ultimi rettangoli di trovino a 'spaziatura' pixel dal bordo destro
        NOTA se la riga contiene un solo rettangolo dovrà essere disegnato centrato in orizzontale
    - ciascuna riga di rettangoli si trova ad una distanza minima di 'spaziatura' pixel in verticale dalla seguente
        NOTA: la distanza in verticale va calcolata tra i due rettangoli più alti delle due linee consecutive
        Il rettangolo più grosso della prima riga si trova a 'spaziatura' pixel dal bordo superiore
        Il rettangolo più grosso dell'ultima riga si trova a 'spaziatura' pixel dal bordo inferiore
    - l'immagine ha le dimensioni minime possibili, quindi:
        - esiste almeno un rettangolo della prima/ultima riga a 'spaziatura' pixel dal bordo superiore/inferiore
        - esiste almeno una riga che ha il primo ed ultimo rettangolo a 'spaziatura' pixel dal bordo sinistro/destro

    NOTA: nel disegnare i rettangoli arrotondate le coordinate che ottenete all'intero più vicino
'''
import images

def ex(file_dati, file_png, spaziatura):
    # leggo i dati come lista di liste di coppie valore,colore
    dati = leggi_dati(file_dati)
    # trovo la larghezza minima necessaria ovvero la riga più larga
    width = larghezza_massima(dati, spaziatura)
    # trovo l'altezza minima necessaria
    height = altezza_minima(dati, spaziatura)
    # costruisco l'immagine
    black = 0,0,0
    img = [ [black]*width for _ in range(height) ]
    # disegno ciascuna riga di valori partendo dalla Y=0
    Y = spaziatura
    for riga in dati:
        # disegno la riga con il quadrato più grosso appoggiato in alto
        # centrata in orizzontale ed aggiorno il bordo superiore
        Y += disegna_riga(img, riga, Y, width, spaziatura)
        # aggiungo la spaziatura sotto
        Y += spaziatura
    # salvo l'immagine
    images.save(img, file_png)
    # torno le dimensioni
    return width, height

def leggi_dati(filename):
    # apro il file
    with open(filename, encoding='utf8') as F:
        dati = []
        # ne leggo le righe e le splitto e converto in liste di interi
        for riga in F:
            riga = riga.replace(',',' ')
            interi = list(map(int, riga.split()))
            # trasformo la riga di interi in riga di cinquine
            cinquine = [ interi[i:i+5] for i in range(0,len(interi),5) ]
            dati.append(cinquine)
            for l,a,r,g,b in cinquine:
                assert l%2==0, 'la larghezza non è pari'
                assert a%2==0, 'la altezza non è pari'
    return dati

def larghezza_massima(dati, spaziatura):
    # torno la massima somma dei valori più la spaziatura
    return max( [ (sum( [ spaziatura+l for l,a,r,g,b in riga ] )+spaziatura) for riga in dati ] )

def altezza_minima(dati, spaziatura):
    # per ogni riga trovo la massima altezza
    massimi = [ max( [ a for l,a,r,g,b in riga ] ) for riga in dati ]
    # ne calcolo la somma più la spaziatura
    N = len(dati)
    return sum(massimi) + spaziatura*(N+1)

def disegna_riga(img, riga, Y, W, spaziatura):
    # la riga deve essere appoggiata in alto alla coordinata Y
    # trovo il massimo ed uso la sua metà+Y come centro verticale di tutti i quadrati
    valori  = [ a for l,a,r,g,b in riga ]
    massimo = max( valori )
    centro  = Y + massimo//2
    # se è un solo valore lo disegno al centro
    N = len(riga)
    if N == 1:
        # FIXME: lo disegno al centro
        for l,a,r,g,b in riga:
            disegna_rettangolo(img, (W//2)-l//2, centro-a//2, l, a, (r,g,b))
    else:
        # calcolo la larghezza della riga ovvero la somma dei valori + spaziatura*(N+1)
        larghezza = sum(spaziatura+l for l,a,r,g,b in riga) + spaziatura
        # calcolo lo spazio rimanente da suddividere
        rimanente = W - larghezza
        # lo distribuisco TRA i rettangoli troncandolo a intero
        assert rimanente%(N-1) == 0, f"lo spazio rimanente {rimanente} non è divisibile per {N-1}"
        dx = rimanente//(N-1)
        # disegno i quadrati appoggiati a X, centrati su Y e distanziati di 100
        X = spaziatura
        for l,a,r,g,b in riga:
            disegna_rettangolo(img, X, centro-a//2, l, a, (r,g,b))
            # aggiorno la x aggiungendo la larghezza del quadrato + 100
            X += l+spaziatura+dx
    return massimo # torno lo spessore della riga di quadrati

def disegna_rettangolo(img, X, Y, W, H, colore):
    for x in range(X,X+W):
        for y in range(Y,Y+H):
            try:
                img[y][x] = colore
            except IndexError:
                pass

if __name__ == '__main__':
    import sys
    name, spaziatura = sys.argv[1:3]
    print(ex(name+'.txt', name+'.png', int(spaziatura)))

