# -*- coding: utf-8 -*-
''' Vogliamo disegnare i dati presenti in un file di testo su un grafico.
    Il file contiene su ciascuna riga, seguiti da 1 virgola e/o 0 o più spazi o accapi,
    gruppi di 5 valori interi che rappresentano per ciascun dato:
        - larghezza
        - alezza
        - canale R del colore
        - canale G del colore
        - canale B del colore
    Ciascuna riga contiene almeno un gruppo di 5 interi positivi relativi ad un dato da plottare.
    Per ciascun dato dobbiamo disegnare un rettangolo del colore indicato e di dimensioni indicate

    Realizzate la funzione ex2(file_dati, file_png, spaziatura) che:
    - legge i dati dal file file_dati
    - costruisce una immagine in formato PNG e la salva nel file file_png
    - ritorna le dimensioni larghezza,altezza dell'immagine

    L'immagine deve avere sfondo nero e visualizzare tutti i dati come segue:
    - l'argomento spaziatura indica il numero di pixel da usare per il bordo esterno
      e la spaziatura minima in orizzontale tra i rettangoli ed in verticale tra le righe
    - ciascun dato è rappresentato da un rettangolo descritto da una quintupla
    - i dati contenuti su ciascuna riga del file devono essere disegnati, centrati verticalmente, su una riga della immagine
    - i rettangoli devono essere equidistanti orizzontalmente l'uno dall'altro
        con una distanza minima di 'spaziatura' pixel tra un rettangolo ed il seguente
        in modo che tutti i primi rettangoli si trovino a 'spaziatura' pixel dal bordo sinistro
        e tutti gli ultimi rettangoli di trovino a 'spaziatura' pixel dal bordo destro
        NOTA se la riga contiene un solo rettangolo dovrà  essere disegnato centrato in orizzontale
    - ciascuna riga di rettangoli si trova ad una distanza minima di 'spaziatura' pixel in verticale dalla seguente
        NOTA: la distanza in verticale va calcolata tra i due rettangoli più alti delle due linee consecutive
        Il rettangolo più grosso della prima riga si trova a 'spaziatura' pixel dal bordo superiore
        Il rettangolo più grosso dell'ultima riga si trova a 'spaziatura' pixel dal bordo inferiore
    - l'immagine ha le dimensioni minime possibili, quindi:
        - esiste almeno un rettangolo della prima/ultima riga a 'spaziatura' pixel dal bordo superiore/inferiore
        - esiste almeno una riga che ha il primo ed ultimo rettangolo a 'spaziatura' pixel dal bordo sinistro/destro

    NOTA: nel disegnare i rettangoli arrotondate le coordinate che ottenete all'intero più vicino
'''
import images

def get_rect_lines(file_dati):
    with open(file_dati) as f:
        lines = map(str.strip, f.readlines())
    return [line.translate({ord(' '):'',ord('\t'):''}) for line in lines if line != '']


def make_rect_lists(rect_lines):
    rects = []
    for line in rect_lines:
        line = line.rstrip(',').split(',')
        rects.append([tuple(map(int,line[i:i+5])) for i in range(0,len(line),5)])
    return rects


def get_width_height(rect_lists, spaziatura):
    h = spaziatura
    for line in rect_lists:
        h += max(line, key = lambda x:x[1])[1] + spaziatura
    w = max(rect_lists, key = lambda x: spaziatura*(len(x)-1) + sum([y[0] for y in x]))
    return spaziatura*(len(w)+1) + sum([y[0] for y in w]), h


def draw_rect(im, row, col, rect):
    row = row-rect[1]//2
    color = tuple(rect[2:])
    rmargin = col+rect[0]
    for r in range(row, row+rect[1]):
        im[r][col:rmargin] = [color]*rect[0]
    return rect[0]


def draw_rect_line(im, upper_row, line, width, spaziatura):
    if len(line) == 1:
        rect = line[0]
        height = rect[1]
        col = width - rect[0]
        draw_rect(im, upper_row + height//2, col//2, rect)
        return rect[1]
    total_length = sum(map(lambda x: x[0], line))
    height = max(line, key = lambda x:x[1])[1]
    fill = width - total_length - 2*spaziatura
    fill = fill//(len(line)-1)
    col = spaziatura
    up = upper_row + height//2
    for rect in line:
        col += draw_rect(im, up, col, rect) + fill
    return height


#%%
def ex(file_dati, file_png, spaziatura):
    rect_lines = get_rect_lines(file_dati)
    rect_lists = make_rect_lists(rect_lines)
    width, height = get_width_height(rect_lists, spaziatura)

    im = [[(0,0,0)]*width for _ in range(height)]
    gap = spaziatura
    for line in rect_lists:
        gap += draw_rect_line(im, gap, line, width, spaziatura) + spaziatura
    images.save(im, file_png)
    return width, height

if __name__ == '__main__':
        print(ex('matrices/mat-8.txt', 'test_mat-8.png', 15))

