import images


def padding(W, n, pad):
    '''
    compute final padded length
    inital lenght W 
    + n_rect*pad
    + final pad
    '''
    return W + (n + 1)*pad


def parse_file(file_path, pad):
    '''
    Get the file_path and padding amount
    Return
    rows - list of rects in quintuple (for each row, r)
    Hs - list of maxium H (for each row , max{Hᵣ})
    sumW - list of sum of rect_i (for each row, ∑Wᵣ, R) 
    total_W, total_H - final image size 
    '''
    with open(file_path, mode='rt', encoding='utf8') as fr:
        rows, Hs, sumW = [], [], []
        for line in fr:
            # get list of ints
            line = list(map(int, line.replace(',', ' ').split()))
            # remap list of ints to list of quintuple
            rows.append([line[i:i+5] for i in range(0, len(line), 5)])
            # rows[-1] is list of rects
            # sum, and max per row
            sum_w, max_h = list(map(lambda f, *values: f(values),
                                    (sum, max), *rows[-1]))
            Hs.append(max_h) # max on H
            sumW.append((sum_w, len(rows[-1]))) # sum widths + num rect
    # sum all max heights
    H = sum(Hs)
    # take max of all summed weights
    Wn = max(sumW, key=lambda T: padding(*T, pad))
    # weight total W+(n+1)*pad
    total_W = padding(*Wn, pad)
    # height total H+(n+1)*pad
    total_H = padding(H, len(Hs), pad)
    return rows, Hs, sumW, total_W, total_H 


def fill_rect(im, x, y, w, h, col):
    colt = tuple(col)
    for delta_y in range(h):
        im[y+delta_y][x:x+w] = [colt]*w


def equal_distance_middle(W, pad, sumw, N):
    '''
    FIX: my first interpretation was like this
    return (W - (2*pad + sumw))/(N-1)
    was working even with this I was occrectly updating
    as x' = x + rect_w + eq_space
    now is:
    Δ = W - ( ∑Wᵣ + R·pad + pad)
    since **at least pad** space between rectangles
    return Δ /(R-1)
    '''
    return (W - padding(sumw, N, pad))//(N-1)


def draw_on_row(final, rects, W, H, pad, sumw, y_current, N, bg=(0, 0, 0)):
    half_H = H//2
    center_y = y_current + half_H
    if N > 1:
        eq_space = equal_distance_middle(W, pad, sumw, N)
        x = pad # we start at pad
        for rect_w, rect_h, *col in rects:
            # |
            # y_current
            # |      rect_h/2
            # half_H |
            # |      |
            y = center_y - rect_h//2
            fill_rect(final, x , y, rect_w, rect_h, col)
            # x' = x + ----rect_w---- + pad + eq-space
            x = x + rect_w + pad + eq_space
    else:
        # we handle single case
        # first rect
        rect_w, rect_h, *col = rects[0]
        y = center_y - rect_h//2
        x = W//2 - rect_w//2
        fill_rect(final, x, y, rect_w, rect_h, col)
    return y_current + H + pad # where we were (y_current) + max_H + bottom pad
            

def get_final_image(W, H, col):
    return [[col]*W for _ in range(H)]


def ex(file_dati, file_png, pad):
    bg = (0, 0, 0)
    # rects in rows, max H per row, sum weights per row, total image size
    rows, Hs, sumW, W_tot, H_tot = parse_file(file_dati, pad)
    # create final image
    final = get_final_image(W_tot, H_tot, bg)
    # start from pad
    y_current = pad
    # drawing rect
    for rects, H, (sumW_rect, n_rect) in zip(rows, Hs, sumW):
        # y_current is updated
        y_current = draw_on_row(final, rects, W_tot, H,
                                pad, sumW_rect, y_current, n_rect)
    # saving and return
    images.save(final, file_png)
    return W_tot, H_tot


if __name__ == '__main__':
    # place here your personal tests
    # id_file, spacing, exp_size, _ = ('example', 42,       (288, 348),      0.5)
    # id_file, spacing, exp_size, _ = ('mat-6',   17,       (440, 393),      0.5)
    id_file, spacing, exp_size, _ =  ('mat-12',  42, (2186, 1101), 1.)
    file_txt     = 'matrices/' + id_file + '.txt'
    file_png     = 'test_'     + id_file + '.png'
    expected_png = 'matrices/' + id_file + '.png'
    assert ex(file_txt, file_png, spacing) == exp_size
