import testlib
import random, os
from ddt import file_data, ddt, data, unpack

# change this variable to True to disable timeout and enable print
DEBUG=False
DEBUG=True

WARP = 2   # test VM
WARP = 1   # your PC

import images   # preload

@ddt
class Test(testlib.TestCase):
    def do_test(self, id_file, spacing, expected, timeout, subdir):
        """Test implementation
        - file_txt:		text file containing the input matrix
        - file_png:	    png image you must write the picture inside
        - spacing:      spacing between rectangles and for the border
        - expected_png:	expected result - file png
        - expected:     expected result - image dimensions
        - timeout:      variable timeout depending on test
        """
        file_txt     = subdir + '/' + id_file + '.txt'
        file_png     = 'test_'      + id_file + '.png'
        expected_png = subdir + '/' + id_file + '.png'
        TIMEOUT = timeout * WARP   # warp factor 
        if DEBUG:
                import program01 as program
                result = program.ex(file_txt, file_png, spacing)
        else:
            with    self.ignored_function('pprint.pprint'), \
                    self.ignore_print(), \
                    self.forbidden_function('builtins.input'), \
                    self.forbidden_function('builtins.eval'), \
                    self.check_open( { file_txt: 'trt', file_png: 'bwb' } ), \
                    self.check_imports(allowed=['program01', '_io', 'images','encodings.utf_8']), \
                    self.timeout(TIMEOUT), \
                    self.timer(TIMEOUT):
                import program01 as program
                result = program.ex(file_txt, file_png, spacing)
        self.assertEqual(type(result), tuple,
                         ('The output type should be: tuple\n'
                          '[Il tipo di dato in output deve essere: tuple]'))
        self.assertEqual(len(result), 2,
                         ('The tuple should contain 2 elements\n'
                          '[La tupla deve contenere due elementi]'))
        self.assertEqual(result, expected,
                         ('The return value is incorrect\n'
                          '[Il valore di ritorno è errato]'))
        self.check_img_file(file_png, expected_png)
        os.remove(file_png)
        return 1

    @data(  # test-id    spacing   exp_dimensions   timeout
            ( 'example',   42,      ( 288, 348),     0.5),
            ( 'minimal',   30,      (  82, 140),     0.5),
            ( 'mat-3-1',    1,      ( 466, 120),     0.5),
            ( 'mat-4-5',    5,      ( 120,  61),     0.5),
            ( 'mat-5-5',    5,      ( 150,  78),     0.5),
            ( 'mat-2-97',  97,      ( 690, 341),     0.5),
            ( 'mat-23-2',   2,      ( 882, 802),     0.5),
            ( 'mat-53-1',   1,      ( 600,1748),     0.5),
            ( 'mat-12-25', 25,      (2362, 813),     1.0),
            ( 'mat-16-25', 25,      (2822,1071),     1.0),
            )
    @unpack
    def test_matrici(self, id_file, spacing, expected, timeout):
        return self.do_test(id_file, spacing, expected, timeout, 'matrices')

    ######################### SECRET TESTS START HERE! #########################

    @data(  # test-id    spacing   exp_dimensions   timeout
            ( 'mat-1-3',    3,      ( 574,  84),     0.5),
            ( 'mat-2-32',  32,      ( 402, 186),     0.5),
            ( 'mat-16-14', 14,      ( 296, 892),     0.5),
            ( 'mat-7-2',    2,      ( 644, 544),     0.5),
            ( 'mat-wide',   9,      (1236, 153),     0.5),
            ( 'mat-3-8',    8,      (5266, 308),     0.5),
            )
    @unpack
    def test_secrets(self, id_file, spacing, expected, timeout):
        return self.do_test(id_file, spacing, expected, timeout, 'secrets')

if __name__ == '__main__':
    Test.main()

